@@ -71,6 +71,8 @@ int opt_test=DEF_TEST;
 int opt_ptrdrag=DEF_PTRDRAG;
 int opt_kill=0;
 int opt_repeater=0, opt_double=0;
+int opt_kernel;
+int opt_explicittype; /* kmouse defaults to autodetection */
 static int opt_resize=0; /* not really an option */
 
 char *prgname;
@@ -183,7 +185,7 @@ static int x1=1, y1=1, x2, y2;
     {
     case GPM_MOVE:
       if (x2<1) x2++; else if (x2>maxx) x2--;
-      if (y2<1) y2++; else if (y2>maxx) y2--;
+      if (y2<1) y2++; else if (y2>maxy) y2--;
       selection_copy(x2,y2,x2,y2,3); /* just highlight pointer */
       return 0;
 
@@ -758,6 +760,16 @@ Gpm_Event event;
   signal(SIGTERM, gpm_killed);
   signal(SIGINT,  gpm_killed);
   signal(SIGUSR1, gpm_killed); /* usr1 is used by a new gpm killing the old */
+
+  if (opt_kernel)
+    while(1)
+      {
+      opt_kill=0;
+      sleep(0xFFFFFF);  /* six months at a time :-) */
+      if (opt_kill)
+	exit(1);
+      }
+
   signal(SIGWINCH,gpm_killed); /* winch can be sent if console is resized */
 
 /*....................................... create your nodes */
@@ -953,6 +965,11 @@ static void gpm_killed(int signo)
     unlink(GPM_NODE_PID);
     unlink(GPM_NODE_CTL);
     }
+  if (opt_kernel)
+    {
+    opt_kill++;
+    return;
+    }
   exit(0);
 }
 
