@@ -60,6 +60,7 @@ typedef struct Gpm_Type {
   int (*fun)(Gpm_Event *state, unsigned char *data);
   struct Gpm_Type *(*init)(int fd, unsigned short flags,
 						   struct Gpm_Type *type);
+  int protoid;            /* for kmouse */
   unsigned short flags;
   unsigned char proto[4];
   int packetlen;
@@ -118,6 +119,7 @@ extern char *opt_lut;
 extern int opt_test, opt_ptrdrag;
 extern int opt_kill;
 extern int opt_repeater, opt_double;
+extern int opt_kernel, opt_explicittype;
 extern int fifofd;
 
 extern Gpm_Type mice[];
