@@ -34,6 +34,7 @@
 #include <sys/types.h>     /* socket() */
 #include <sys/socket.h>    /* socket() */
 #include <sys/un.h>        /* struct sockaddr_un */
+#include <linux/fs.h>      /* MAJOR */
 
 #ifdef	SIGTSTP		/* true if BSD system */
 #include <sys/file.h>
@@ -43,12 +44,16 @@
 #include "gpmInt.h"
 #include "gpm.h"
 
+#include "kmouse.h"  /* kmouse ioctl */
+
 #define mkfifo(path, mode) (mknod ((path), (mode) | S_IFIFO, 0))
 
 extern int	errno;
 
 static int check_uniqueness(int retain);
 
+static struct kmouse_options kopt;
+
 /*===================================================================*
  *     ==>    the logging utility
  *
@@ -229,10 +234,11 @@ static int usage(char *whofailed)
  "    -B sequence      allows changing the buttons (default '%s')\n"
  "    -d delta         sets the delta value (default %d)\n"
  "    -D               dirty operation (debug only)\n"
- "    -g tap-button    allows setting the button that is emulated by tapping\n"
- "                     on a glidepoint mouse\n"
+ "    -g tap-button    sets the button (1-3) that is emulated by tapping on\n"
+ "                     a glidepoint mouse, none by default. (mman/ps2 only)\n"
  "    -i interval      max time interval for multiple clicks (default %i)\n"
  "    -k               kill a running gpm, to start X with a busmouse\n"
+ "    -K               run in kernel mode, using the kmouse module\n"
  "    -l charset       loads the inword() LUT (default '%s')\n"
  "    -L charset       load LUT and exit\n"
  "    -m mouse-device  sets mouse device\n"
@@ -240,7 +246,6 @@ static int usage(char *whofailed)
  "                     the second device. Forces \"-R\"\n"
  "    -o modem-lines   toggle modem lines (\"dtr\", \"rts\", \"both\")\n"
  "    -p               draw the pointer while striking a selection\n"
- "    -P               do it only with word and line drags\n"
  "    -q               quit after changing mouse behaviour\n"
  "    -r number        sets the responsiveness (default %i)\n"
  "    -R               enter repeater mode. X should read /dev/gpmdata\n"
@@ -272,7 +277,7 @@ static int check_uniqueness(int retain)
   if ( (ctlfd=socket(AF_UNIX,SOCK_STREAM,0))<0 )
 	oops("socket()");
 
-  if ( connect(ctlfd,(struct sockaddr *)(&ctladdr),len)>=0 )
+  if ( connect(ctlfd,(struct sockaddr *)(&ctladdr),len)>=0 || opt_kill)
 	{
 	if (retain) return ctlfd;
 	/* another gpm is runnin, get its pid */
@@ -314,8 +319,9 @@ return 0; /* never */
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dg:hi:kl:L:m:Mo:pqr:Rs:t:Tv23";
-int i, opt, fd;
+char options[]="a:b:B:d:Dg:hi:Kkl:L:m:Mo:pqr:Rs:t:Tv23";
+int i, opt, fd, kfd;
+struct kmouse_options koptions;
 FILE *f;
 static struct {char *in; char *out;} seq[] = {
   {"123","01234567"},
@@ -345,6 +351,7 @@ static struct {char *in; char *out;} seq[] = {
       case 'h': exit(usage(NULL));
       case 'i': opt_time=atoi(optarg); break;
       case 'k': opt_kill++; break;
+      case 'K': opt_kernel++; break;
       case 'l': opt_lut=optarg; break;
       case 'L': opt_lut=optarg; opt_quit++; break;
       case 'm': opt_dev = optarg; break;
@@ -354,6 +361,7 @@ static struct {char *in; char *out;} seq[] = {
           else if (!strcmp(optarg,"rts"))  opt_toggle=TIOCM_RTS;
           else if (!strcmp(optarg,"both")) opt_toggle=TIOCM_DTR | TIOCM_RTS;
           else exit(usage("toggle lines"));
+          break;
       case 'p': opt_ptrdrag=0; break;
       case 'q': opt_quit++; break;
       case 'r':
@@ -364,7 +372,7 @@ static struct {char *in; char *out;} seq[] = {
 	  break;
       case 'R': opt_repeater++; break;
       case 's': opt_sample = atoi(optarg); break;
-      case 't': opt_type=optarg; break;
+      case 't': opt_type=optarg; opt_explicittype++; break;
       case 'T': opt_test++; break;
       case 'v': printf(GPM_NAME " " GPM_RELEASE ", " GPM_DATE "\n"); exit(0);
       case '2': opt_three=-1; break;
@@ -398,9 +406,10 @@ static struct {char *in; char *out;} seq[] = {
     if (opt_delta<2) exit(usage("delta"));
     if (strlen(opt_sequence)!=3 || atoi(opt_sequence)<100)
       exit(usage("sequence"));
-    if(opt_glidepoint_tap<1 || opt_glidepoint_tap>3)
+    if (opt_glidepoint_tap>3)
       exit(usage("glidepoint tap button"));
-    opt_glidepoint_tap=GPM_B_LEFT >> (opt_glidepoint_tap-1);
+    if (opt_glidepoint_tap)
+      opt_glidepoint_tap=GPM_B_LEFT >> (opt_glidepoint_tap-1);
 
     /* choose the sequence */
     for (opt=0; seq[opt].in && strcmp(seq[opt].in,opt_sequence); opt++)
@@ -474,10 +483,13 @@ static struct {char *in; char *out;} seq[] = {
 #endif
 
   /* chdir */
-  if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
-    oops(GPM_NODE_DIR);
-  if (chdir(GPM_NODE_DIR))
-    oops(GPM_NODE_DIR);      /* well, I must create my directory first */
+  if (!opt_kernel) /* FIXME -- this is to find kmouse.o */
+    {
+    if (chdir(GPM_NODE_DIR) && mkdir(GPM_NODE_DIR,GPM_NODE_DIR_MODE))
+      oops(GPM_NODE_DIR);
+    if (chdir(GPM_NODE_DIR))
+      oops(GPM_NODE_DIR);      /* well, I must create my directory first */
+    }
 
   /* now sign */
   f=fopen(GPM_NODE_PID,"w");
@@ -498,7 +510,96 @@ static struct {char *in; char *out;} seq[] = {
    * symlink(GPM_NODE_CTL,"/tmp/gpmctl");
    */
 
-  return mouse_table[1].fd; /* the second is handled in the main() */
+  if (!opt_kernel)
+    return mouse_table[1].fd; /* the second is handled in the main() */
+
+  /*
+   * Everything from here onwards is setting up for kmouse
+   */
+
+#if 0 /* don't actively call insmod, let kerneld do it */
+  switch(fork()) /* insert the module */
+    {
+    case -1: 
+      fprintf(stderr,"%s: fork(): %s\n",prgname,strerror(errno));
+      exit(1);
+    case 0:
+      execl("/sbin/insmod","insmod","kmouse",NULL);
+      execlp("insmod","insmod","kmouse",NULL);
+      fprintf(stderr,"insmod: %s\n",strerror(errno));
+      exit(1);
+    default:
+      wait(&i);
+    }
+
+  /* 
+   * Unfortunately, insmod doesn't tell why loading fails.
+   * If the module is already there, it's all right. So go on.
+   */
+  if (i)
+    fprintf(stderr,"%s: Warning: insmod failed\n", prgname);
+#endif
+
+  if ( (kfd=open("/dev/gpmctl",O_RDONLY)) < 0 )
+    {
+    fprintf(stderr,"%s: /dev/gpmctl: %s\n", prgname, strerror(errno));
+    exit(1);
+    }
+  if (ioctl(kfd,KMOUSE_GETVER,&i)<0)
+    {
+    fprintf(stderr,"%s: ioctl(/dev/gpmctl): %s\n", prgname, strerror(errno));
+    exit(1);
+    }
+  if (KMOUSE_V_MAJOR(i) != KMOUSE_V_MAJOR(KMOUSE_VERSION_N))
+    {
+    fprintf(stderr,"%s: kmouse version %i.%i not supoorted: %i.xx needed\n",
+	    KMOUSE_V_MAJOR(i),KMOUSE_V_MINOR(i),
+	    KMOUSE_V_MAJOR(KMOUSE_VERSION_N));
+    exit(1);
+    }
+  for (i=1; i <= 1+opt_double; i++)
+    {
+    struct stat buffer;
+    which_mouse=mouse_table+i; /* used to access options */
+    
+    if (stat(opt_dev,&buffer)<0)
+      {
+      fprintf(stderr,"%s: stat(%s): %s\n",prgname,opt_dev,
+	      strerror(errno));
+      exit(1);
+      }
+    koptions.protoid = opt_explicittype ? m_type->protoid : PROTO_UNKNOWN;
+    strncpy(koptions.sequence,opt_sequence,8);
+    koptions.dev=buffer.st_rdev;
+    koptions.three=opt_three;
+    koptions.delta=opt_delta;
+    koptions.accel=opt_accel;
+    koptions.scale=opt_scale;
+    koptions.scaley=opt_scaley; /* FIXME */
+    koptions.time=opt_time;
+    koptions.tap=opt_glidepoint_tap;
+    koptions.ptrdrag=opt_ptrdrag;
+    if (ioctl(kfd,KMOUSE_SETOPT,&koptions)<0)
+      {
+      fprintf(stderr,"%s: ioctl(/dev/gpmctl): %s\n",prgname,strerror(errno));
+      exit(1);
+      }
+    i=MAJOR(buffer.st_dev);     /* compare with another serial port */
+    stat("/dev/ttyS0",&buffer);
+    if (MAJOR(buffer.st_dev)==i)
+      {
+      i=N_MOUSE;
+      if (ioctl(which_mouse->fd,TIOCSETD,&i)<0) 
+	{
+	fprintf(stderr,"%s: Warning: ioctl(TIOCSETD) failed for %s: %s\n",
+		prgname,opt_dev,strerror(errno));
+	}
+      }
+    }
+  close(kfd);
+  chdir("/");
+  return -1; /* no fd to be used */
+  
 }
   
 
