@@ -52,6 +52,7 @@
 #include <errno.h>
 
 #include "gpmInt.h"
+#include "kmouse.h"     /* protocol id's */
 
 /*========================================================================*/
 /*
@@ -83,18 +84,16 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
   state->dx=      (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
   state->dy=      (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
 
-  if (state->dx || state->dy) /* motion: preserve buttons */
-	state->buttons=prev;
+  /* Allow motion *and* button change (Michael Plass) */
+
+  if ((state->dx==0) && (state->dx==0) 
+      && (state->buttons == (prev&~GPM_B_MIDDLE)))
+    state->buttons = prev^GPM_B_MIDDLE;  /* no move or change: toggle middle */
   else
-	{
-	if (state->buttons == (prev&~GPM_B_MIDDLE))
-	  state->buttons = prev^GPM_B_MIDDLE;     /* no change: toggle middle */
-    else
-	  state->buttons |= prev&GPM_B_MIDDLE;    /* change: preserve middle */
-	
-	}
+    state->buttons |= prev&GPM_B_MIDDLE;    /* change: preserve middle */
   prev=state->buttons;
 
+
 #endif
 
   return 0;
@@ -151,11 +150,20 @@ static int M_bm(Gpm_Event *state,  unsigned char *data) /* equal to sun */
 
 static int M_ps2(Gpm_Event *state,  unsigned char *data)
 {
+  static int tap_active=0; /* there exist glidepoint ps2 mice */
+
   state->buttons=
     !!(data[0]&1) * GPM_B_LEFT +
     !!(data[0]&2) * GPM_B_RIGHT +
     !!(data[0]&4) * GPM_B_MIDDLE;
 
+  if (data[0]==0 && opt_glidepoint_tap) /* by default this is false */
+    state->buttons = tap_active = opt_glidepoint_tap;
+  else if (tap_active)
+    if (data[0]==8)
+      state->buttons = tap_active = 0;
+    else state->buttons = tap_active;
+
   state->dx=   (data[0] & 0x10) ? data[1]-256 : data[1];
   state->dy= -((data[0] & 0x20) ? data[2]-256 : data[2]);
   return 0;
@@ -219,6 +227,36 @@ static int M_mman(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
+/* wacom tablet, in native mode */
+static int M_wacom(Gpm_Event *state, unsigned char *data)
+{
+  static int ox=-1, oy, threshold=8;
+  int x, y, z, p;
+
+  if (data[0]==0xa0) /* leave -- but sometimes it's not decoded */
+    { ox=-1; state->buttons=0; state->dx=state->dy=0; return 0;}
+
+  
+  x = (data[1]<<7) | (data[2]&0x7f);
+  y = (data[4]<<7) | (data[5]&0x7f);
+  z = data[6]-64 + 128*(data[6]<64);
+  if (ox==-1 || abs(x-ox)>128 || abs(y-oy)>128) { ox=x; oy=y;} /* enter */
+
+  state->buttons = (GPM_B_MIDDLE * !!(data[3]&0x10)); /* the b. means middle */
+  if (!state->buttons)
+    if (data[3]&0x20) state->buttons = (z>threshold) * GPM_B_RIGHT;
+    else              state->buttons = (z>threshold) * GPM_B_LEFT;
+
+  if (z>threshold) threshold=1;
+  else threshold=8;
+
+  state->dx = (x-ox)/5; 
+  state->dy = (y-oy)/5; /* divide by 5, 'cause it's too much sensitive */
+
+  ox=x; oy=y;
+  return 0;
+}
+
 /* ncr pen support (Marc Meis) */
 
 #define NCR_LEFT_X     40
@@ -405,6 +443,17 @@ int busmouse;
   return type;
 }
 
+static Gpm_Type *I_wacom(int fd, unsigned short flags, struct Gpm_Type *type)
+{
+  int i;
+
+  if (opt_baud=1200) opt_baud=9600; /* default to 9600 */
+  /* change from any available speed to the chosen one */
+  for (i=9600; i>=1200; i/=2)
+    setspeed(fd, i, opt_baud, flags);
+  return type;
+}
+
 
 /*========================================================================*/
 /* Finally, the table */
@@ -416,37 +465,42 @@ int busmouse;
  */
 Gpm_Type mice[]={
   {"mman", "The \"MouseMan\" and similar devices (3/4 bytes per packet).",
-           "Mouseman", M_mman, I_serial, CS7 | STD_FLG, /* MUST BE FIRST */
+           "Mouseman", M_mman, I_serial, PROTO_MMAN, CS7 | STD_FLG, /* first */
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 1, 0},
   {"ms",   "The original micro$oft protocol, with a middle-button extension.",
-           "", M_ms, I_serial, CS7 | STD_FLG,
+           "", M_ms, I_serial, PROTO_MS, CS7 | STD_FLG,
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
   {"bare", "The ms protocol, unextended. Necessary for some 2-buttons mice.",
-           "Microsoft", M_bare, I_serial, CS7 | STD_FLG,
+           "Microsoft", M_bare, I_serial, PROTO_BARE, CS7 | STD_FLG,
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
   {"msc",  "Mouse-Systems-Compatible. Used in most 3-button mice (5-bytes).",
-           "MouseSystems", M_msc, I_serial, CS8 | CSTOPB | STD_FLG,
+           "MouseSystems", M_msc, I_serial, PROTO_MSC, CS8 | CSTOPB | STD_FLG,
                                 {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0, 0},
   {"sun",  "'msc' protocol, but only 3 bytes per packet.",
-           "", M_sun, I_serial, CS8 | CSTOPB | STD_FLG,
+           "", M_sun, I_serial, PROTO_SUN, CS8 | CSTOPB | STD_FLG,
                                 {0xf8, 0x80, 0x00, 0x00}, 3, 1, 0, 0},
   {"mm",   "MM series. Probably an old protocol...",
-           "MMSeries", M_mm, I_serial, CS8 | PARENB|PARODD | STD_FLG,
+           "MMSeries", M_mm, I_serial, PROTO_MM, CS8 | PARENB|PARODD | STD_FLG,
                                 {0xe0, 0x80, 0x80, 0x00}, 3, 1, 0, 0},
   {"logi", "Used in some Logitech devices (only serial).",
-           "Logitech", M_logi, I_logi, CS8 | CSTOPB | STD_FLG,
+           "Logitech", M_logi, I_logi, PROTO_LOGI, CS8 | CSTOPB | STD_FLG,
                                 {0xe0, 0x80, 0x80, 0x00}, 3, 3, 0, 0},
   {"bm",   "Micro$oft busmice and compatible devices.",
-           "BusMouse", M_bm, NULL,  STD_FLG,
+           "BusMouse", M_bm, NULL,  PROTO_SUN, STD_FLG, /* bm is sun */
                                 {0xf8, 0x80, 0x00, 0x00}, 3, 3, 0, 0},
   {"ps2",  "Busmice of the ps/2 series. Most busmice, actually.",
-           "PS/2", M_ps2, NULL,  STD_FLG,
+           "PS/2", M_ps2, NULL,  PROTO_PS2, STD_FLG,
                                 {0xc0, 0x00, 0x00, 0x00}, 3, 1, 0, 0},
   {"ncr",  "Ncr3125pen, found on some laptops",
-           "", M_ncr, NULL, STD_FLG,
+           "", M_ncr, NULL, PROTO_NCR, STD_FLG,
                                 {0x08, 0x08, 0x00, 0x00}, 7, 7, 0, 1},
+  {"wacom",  "Wacom tablet",
+           "", M_wacom, NULL, PROTO_WACOM, STD_FLG,
+                                {0x80, 0x80, 0x80, 0x00}, 7, 7, 0, 0},
 
-  {"", "", "", NULL, NULL, 0,   {0x00, 0x00, 0x00, 0x00}, 0, 0, 0, 0}
+  {"",     "",
+           "", NULL, NULL, PROTO_UNKNOWN, 0,
+                                {0x00, 0x00, 0x00, 0x00}, 0, 0, 0, 0}
 };
 
 /*------------------------------------------------------------------------*/
