@@ -48,8 +48,8 @@
 #endif
 
 #define RMEV_NAME "Mev"
-#define RMEV_RELEASE "1.06"
-#define RMEV_DATE "November 1995"
+#define RMEV_RELEASE "1.09"
+#define RMEV_DATE "May 1996"
 
 
 /*===================================================================*/
@@ -63,7 +63,8 @@ int user_handler(Gpm_Event *event, void *data)
 	   event->x, event->y,
 	   event->dx, event->dy,
 	   event->buttons, event->modifiers);
-
+    fflush(stdout);
+    
     event->dx=event->dy=0; /* prepare for repetitions */
     if (event->type&GPM_DOWN) event->type=GPM_DRAG;
     }
@@ -119,6 +120,7 @@ char cmd[128];
     if (c<' ') d+=0x40;
     if (c==0x7F) d='?';
     printf("key: %02X ('%s%c')\r\n",c,s,d);
+    fflush(stdout);
     }
 
   noraw();
