@@ -53,7 +53,10 @@ by 't-mouse.el")
       (goto-char 0)
       (or
        (re-search-forward "p \\([0-9a-f]\\)" nil t)
-       (re-search-forward "v0\\([0-9a-f]\\)" nil t))
+       (re-search-forward "v0\\([0-9a-f]\\)" nil t)
+       ;; Hmmmm... ps-0.99 changed it again. No key letter any more
+       ;; I must look for the second field...
+       (re-search-forward "[0-9]+ +\\([0-9]+\\)" nil t))
       (and
        (match-beginning 1)
        (setq tty (buffer-substring (match-beginning 1) (match-end 1)))))
